<?php
/**
 * SPFWC_Payment_Response_Handler class.
 *
 * @package Sony Payment Services pro for WooCommerce
 * @since 1.0.0
 */

if ( ! defined( 'ABSPATH' ) ) {
	exit;
}

class SPFWC_Payment_Response_Handler {

	/**
	 * Constructor.
	 */
	public function __construct() {
		add_action( 'woocommerce_api_wc_sonypayment', array( $this, 'response_handler' ) );
	}

	/**
	 * Response handler.
	 */
	public function response_handler() {

		if ( ! isset( $_GET['wc-api'] ) || ( 'wc_sonypayment' !== wp_unslash( $_GET['wc-api'] ) ) ) {
			return;
		}

		try {
			// Payment response
			if ( $this->is_valid_payment_notification() ) {
				$settings = get_option( 'woocommerce_sonypayment_cvs_settings', array() );
				if ( $settings['merchant_id'] === wp_unslash( $_REQUEST['MerchantId'] ) ) {
					$response_data = wp_unslash( $_REQUEST );

					$order_id = absint( $response_data['MerchantFree2'] );
					$order    = wc_get_order( $order_id );

					if ( isset( $response_data['CvsCd'] ) ) {
						$cvs_name = spfwc_get_cvs_name( $response_data['CvsCd'] );
						$message  = sprintf( __( 'Payment completed in %s.', 'woo-sonypayment' ), $cvs_name );
					} else {
						$message = __( 'Payment is completed.', 'woo-sonypayment' );
					}
					$order->update_status( 'processing', $message );

					$trans_code                 = $response_data['MerchantFree1'];
					$response_data['OperateId'] = 'paid';
					SPFWC_Payment_Logger::add_log( $response_data, $order_id, $trans_code );
				}

				header( 'HTTP/1.1 200 OK' );
				exit;

			} elseif ( isset( $_REQUEST['EncryptValue'] ) ) {
				$settings = get_option( 'woocommerce_sonypayment_settings', array() );

				// 3D Secure Authentication response
				if ( 'yes' !== $settings['linktype'] && 'yes' === $settings['three_d_secure'] ) {
					$encrypt_value = SPFWC_SLN_Connection::get_decrypt_value_3dsecure( wp_unslash( $_REQUEST['EncryptValue'] ) );
					if ( ! $encrypt_value ) {
						throw new SPFWC_Exception( 'EncryptValue is empty.' );
					}

					parse_str( $encrypt_value, $response_data );
					if ( ! $this->is_valid_3dsecure_response( $response_data ) ) {
						throw new SPFWC_Exception( 'EncryptValue is invalid.' );
					}

					// Redirect to error page when authentication fails on (1) (8) (9).
					if ( 1 == $response_data['SecureResultCode'] || 8 == $response_data['SecureResultCode'] || 9 == $response_data['SecureResultCode'] ) {
						$responsecd = explode( '|', $response_data['ResponseCd'] );
						foreach ( (array) $responsecd as $cd ) {
							$response_data[ $cd ] = SPFWC_Payment_Message::response_message( $cd );
						}
						$localized_message = __( '3D Secure authentication failed.', 'woo-sonypayment' );
						throw new SPFWC_Exception( print_r( $response_data, true ), $localized_message );
					}

					// $post3d = WC()->session->get( 'post3d' );
					$post3d = SPFWC_Payment_Logger::get_post_log( $response_data['MerchantFree1'] );
					if ( $post3d ) :
						// WC()->session->__unset( 'post3d' );
						SPFWC_Payment_Logger::clear_post_log( $response_data['MerchantFree1'] );
						?>
						<!DOCTYPE html>
						<html lang="ja">
						<head>
						<title></title>
						</head>
						<body onload="javascript:document.forms['redirectForm'].submit();">
						<form action="<?php echo esc_url( wc_get_checkout_url() ); ?>" method="post" id="redirectForm">
						<?php
						// Revert POST values.
						foreach ( (array) $post3d as $key => $value ) {
							if ( is_array( $value ) ) {
								foreach ( (array) $value as $k => $v ) {
									echo '<input type="hidden" name="' . $key . '[' . $k . ']" value="' . $v . '" />' . "\n";
								}
							} else {
								echo '<input type="hidden" name="' . $key . '" value="' . $value . '" />' . "\n";
							}
						}

						// Send 3D Secure Authentication results.
						if ( isset( $response_data['EncodeXId3D'] ) ) {
							echo '<input type="hidden" name="EncodeXId3D" value="' . $response_data['EncodeXId3D'] . '" />' . "\n";
							echo '<input type="hidden" name="TransactionId3D" value="' . base64_decode( $response_data['EncodeXId3D'] ) . '" />' . "\n";
						}
						if ( isset( $response_data['MessageVersionNo3D'] ) ) {
							echo '<input type="hidden" name="MessageVersionNo3D" value="' . $response_data['MessageVersionNo3D'] . '" />' . "\n";
						}
						if ( isset( $response_data['TransactionStatus3D'] ) ) {
							echo '<input type="hidden" name="TransactionStatus3D" value="' . $response_data['TransactionStatus3D'] . '" />' . "\n";
						}
						if ( isset( $response_data['CAVVAlgorithm3D'] ) ) {
							echo '<input type="hidden" name="CAVVAlgorithm3D" value="' . $response_data['CAVVAlgorithm3D'] . '" />' . "\n";
						}
						if ( isset( $response_data['ECI3D'] ) ) {
							echo '<input type="hidden" name="ECI3D" value="' . $response_data['ECI3D'] . '" />' . "\n";
						}
						if ( isset( $response_data['CAVV3D'] ) ) {
							echo '<input type="hidden" name="CAVV3D" value="' . $response_data['CAVV3D'] . '" />' . "\n";
						}
						if ( isset( $response_data['SecureResultCode'] ) ) {
							echo '<input type="hidden" name="SecureResultCode" value="' . $response_data['SecureResultCode'] . '" />' . "\n";
						}
						?>
						<?php // 3D Secure Authentication complete. ?>
						<input type="hidden" name="done3d" value="1" />
						<div class="wait_message" style="text-align: center; margin-top: 100px;"><?php esc_html_e( __( 'Please wait a moment.', 'woo-sonypayment' ) ); ?></div>
						</form>
						</body>
						</html>
						<?php
					else :
						$localized_message = __( 'Failed to return from 3D secure authentication.', 'woo-sonypayment' );
						throw new SPFWC_Exception( print_r( $response_data, true ), $localized_message );
					endif;
					exit;

				// Link type response
				} else {
					$encrypt_value = SPFWC_SLN_Connection::get_decrypt_value( wp_unslash( $_REQUEST['EncryptValue'] ) );
					if ( ! $encrypt_value ) {
						throw new SPFWC_Exception( 'EncryptValue is empty.' );
					}

					parse_str( $encrypt_value, $response_data );
					if ( ! $this->is_valid_payment_response( $response_data ) ) {
						throw new SPFWC_Exception( 'EncryptValue is invalid.' );
					}

					// From checkout page.
					if ( isset( $response_data['MerchantFree2'] ) ) {

						// Cancel
						$cancel = array( 'P51', 'P52', 'P55', 'P56', 'P62', 'P63', 'P64', 'P65', 'P69', 'P70' );
						if ( in_array( $response_data['ResponseCd'], $cancel ) ) {
							$return_url = wc_get_checkout_url();

							// Return checkout page redirect.
							wp_redirect( $return_url );

						// Success
						} elseif ( 'OK' === $response_data['ResponseCd'] ) {

							$order_id   = absint( $response_data['MerchantFree2'] );
							$order      = wc_get_order( $order_id );
							$return_url = $order->get_checkout_order_received_url();

							if ( '4MemAdd' === $response_data['OperateId'] || '4MemChg' === $response_data['OperateId'] ) {
								$customer_id = absint( $response_data['MerchantFree3'] );
								$member      = new SPFWC_Card_Member( $customer_id );
								$member->set_member_id( $response_data['KaiinId'] );
								$member->set_member_pass( $response_data['KaiinPass'] );

								$settings   = get_option( 'woocommerce_sonypayment_settings', array() );
								$operate_id = $settings['operate_id'];

								// Payment process
								$sln                           = new SPFWC_SLN_Connection();
								$params                        = array();
								$param_list                    = array();
								$param_list['MerchantId']      = $settings['merchant_id'];
								$param_list['MerchantPass']    = $settings['merchant_pass'];
								$param_list['TenantId']        = $settings['tenant_id'];
								$param_list['TransactionDate'] = $response_data['TransactionDate'];
								$param_list['MerchantFree1']   = $response_data['MerchantFree1'];
								$param_list['MerchantFree2']   = $response_data['MerchantFree2'];
								$param_list['MerchantFree3']   = $response_data['MerchantFree3'];
								$param_list['KaiinId']         = $response_data['KaiinId'];
								$param_list['KaiinPass']       = $response_data['KaiinPass'];
								$param_list['OperateId']       = $operate_id;
								$param_list['PayType']         = '01';
								$param_list['Amount']          = $order->get_total();
								if ( isset( $response_data['EncodeXId3D'] ) ) {
									$param_list['EncodeXId3D'] = wp_unslash( $response_data['EncodeXId3D'] );
								}
								if ( isset( $response_data['TransactionId3D'] ) ) {
									$param_list['TransactionId3D'] = wp_unslash( $response_data['TransactionId3D'] );
								}
								if ( isset( $response_data['MessageVersionNo3D'] ) ) {
									$param_list['MessageVersionNo3D'] = wp_unslash( $response_data['MessageVersionNo3D'] );
								}
								if ( isset( $response_data['TransactionStatus3D'] ) ) {
									$param_list['TransactionStatus3D'] = wp_unslash( $response_data['TransactionStatus3D'] );
								}
								if ( isset( $response_data['CAVVAlgorithm3D'] ) ) {
									$param_list['CAVVAlgorithm3D'] = wp_unslash( $response_data['CAVVAlgorithm3D'] );
								}
								if ( isset( $response_data['ECI3D'] ) ) {
									$param_list['ECI3D'] = wp_unslash( $response_data['ECI3D'] );
								}
								if ( isset( $response_data['CAVV3D'] ) ) {
									$param_list['CAVV3D'] = wp_unslash( $response_data['CAVV3D'] );
								}
								if ( isset( $response_data['SecureResultCode'] ) ) {
									$param_list['SecureResultCode'] = wp_unslash( $response_data['SecureResultCode'] );
								}

								$params['param_list'] = $param_list;
								$params['send_url']   = $sln->send_url();
								$response_payment     = $sln->connection( $params );
								if ( 'OK' !== $response_payment['ResponseCd'] ) {
									$responsecd = explode( '|', $response_payment['ResponseCd'] );
									foreach ( (array) $responsecd as $cd ) {
										$response_payment[ $cd ] = SPFWC_Payment_Message::response_message( $cd );
									}
									$localized_message = __( 'Payment processing failed. Please retry.', 'woo-sonypayment' );
									throw new SPFWC_Exception( print_r( $response_payment, true ), $localized_message );
								}

								$trans_code = $response_payment['MerchantFree1'];
								update_post_meta( $order_id, '_spfwc_trans_code', $trans_code );

								SPFWC_Payment_Logger::add_log( $response_payment, $order_id, $trans_code );

								$order->payment_complete( $trans_code );

								$message = __( 'Payment is completed.', 'woo-sonypayment' );
								$order->add_order_note( $message );

								if ( is_callable( array( $order, 'save' ) ) ) {
									$order->save();
								}

								// Remove cart.
								WC()->cart->empty_cart();

								// Return thank you page redirect.
								wp_redirect( $return_url );

							} else {
								$trans_code = $response_data['MerchantFree1'];
								update_post_meta( $order_id, '_spfwc_trans_code', $trans_code );

								SPFWC_Payment_Logger::add_log( $response_data, $order_id, $trans_code );

								$order->payment_complete( $trans_code );

								$message = __( 'Payment is completed.', 'woo-sonypayment' );
								$order->add_order_note( $message );

								if ( is_callable( array( $order, 'save' ) ) ) {
									$order->save();
								}

								// Remove cart.
								WC()->cart->empty_cart();

								// Return thank you page redirect.
								wp_redirect( $return_url );
							}

						// Error
						} else {
							$responsecd = explode( '|', $response_data['ResponseCd'] );
							foreach ( (array) $responsecd as $cd ) {
								$response_data[ $cd ] = SPFWC_Payment_Message::response_message( $cd );
							}
							$localized_message = __( 'Payment processing failed. Please retry.', 'woo-sonypayment' );
							throw new SPFWC_Exception( print_r( $response_data, true ), $localized_message );
						}

					// From myaccount page.
					} else {
						if ( '4MemAdd' === $response_data['OperateId'] || '4MemChg' === $response_data['OperateId'] ) {
							if ( 'OK' !== $response_data['ResponseCd'] ) {
								$responsecd = explode( '|', $response_data['ResponseCd'] );
								foreach ( (array) $responsecd as $cd ) {
									$response_data[ $cd ] = SPFWC_Payment_Message::response_message( $cd );
								}
								$localized_message = ( '4MemAdd' === $response_data['OperateId'] ) ? __( 'Failed saving card member.', 'woo-sonypayment' ) : __( 'Card member update failed.', 'woo-sonypayment' );
								throw new SPFWC_Exception( print_r( $response_data, true ), $localized_message );
							}

							$customer_id = absint( $response_data['MerchantFree3'] );
							$member      = new SPFWC_Card_Member( $customer_id );
							$member->set_member_id( $response_data['KaiinId'] );
							$member->set_member_pass( $response_data['KaiinPass'] );
							wp_safe_redirect( wc_get_page_permalink( 'myaccount' ) );
						}
					}
					exit;
				}
			}

		} catch ( Exception $e ) {
			wc_add_notice( $e->getLocalizedMessage(), 'error' );
			SPFWC_Logger::add_log( 'Error: ' . $e->getMessage() );

			wp_die(
				$e->getMessage(),
				'Bad request',
				array(
					'response' => 400,
				)
			);
		}
	}

	/**
	 * Checks if required items are set.
	 *
	 * @return bool
	 */
	public function is_valid_payment_notification() {

		if ( ! isset( $_REQUEST['MerchantId'] ) ||
			! isset( $_REQUEST['TransactionId'] ) ||
			! isset( $_REQUEST['RecvNum'] ) ||
			! isset( $_REQUEST['NyukinDate'] ) ||
			! isset( $_REQUEST['MerchantFree1'] ) ||
			! isset( $_REQUEST['MerchantFree2'] ) ) {
			return false;
		}

		return true;
	}

	/**
	 * Checks if required items are set.
	 *
	 * @return bool
	 */
	public function is_valid_3dsecure_response( $response_data = null ) {

		if ( null === $response_data ) {
			return false;
		}

		if ( ! isset( $response_data['SecureResultCode'] ) ||
			! isset( $response_data['ResponseCd'] ) ) {
			return false;
		}

		return true;
	}

	/**
	 * Checks if required items are set.
	 *
	 * @return bool
	 */
	public function is_valid_payment_response( $response_data = null ) {

		if ( null === $response_data ) {
			return false;
		}

		if ( ! isset( $response_data['MerchantId'] ) ||
			! isset( $response_data['OperateId'] ) ||
			! isset( $response_data['ResponseCd'] ) ||
			! isset( $response_data['MerchantFree1'] ) ) {
			return false;
		}

		$settings = get_option( 'woocommerce_sonypayment_settings', array() );
		if ( $response_data['MerchantId'] !== $settings['merchant_id'] ) {
			return false;
		}

		return true;
	}
}

new SPFWC_Payment_Response_Handler();
